@@ -1,6 +1,6 @@
 #' @name make_surrogate_data
 #' @aliases make_surrogate_shuffle make_surrogate_ebisuzaki 
-#'   make_surrogate_seasonal
+#'   make_surrogate_seasonal make_surrogate_twin
 #' 
 #' @title Generate surrogate data for permutation/randomization tests
 #'
@@ -48,6 +48,11 @@ make_surrogate_data <- function(ts, method = c("random_shuffle", "ebisuzaki",
 #' 
 make_surrogate_shuffle <- function(ts, num_surr = 100)
 {
+    if (is.data.frame(ts))
+    {
+        ts <- ts[[1]]
+    }
+    
     matrix(unlist(
         lapply(seq(num_surr), function(i) {
             sample(ts, size = length(ts))
@@ -68,6 +73,11 @@ make_surrogate_shuffle <- function(ts, num_surr = 100)
 #' 
 make_surrogate_ebisuzaki <- function(ts, num_surr = 100)
 {
+    if (is.data.frame(ts))
+    {
+        ts <- ts[[1]]
+    }
+    
     if (any(!is.finite(ts)))
         stop("input time series contained invalid values")
     
@@ -81,24 +91,24 @@ make_surrogate_ebisuzaki <- function(ts, num_surr = 100)
     
     matrix(unlist(
         lapply(seq(num_surr), function(i) {
-        if (n %% 2 == 0) # even length
-        {
-            thetas <- 2 * pi * runif(n2 - 1)
-            angles <- c(0, thetas, 0, -rev(thetas))
-            recf <- amplitudes * exp(complex(imaginary = angles))
-            recf[n2] <- complex(real = sqrt(2) * amplitudes[n2] * 
-                                    cos(runif(1) * 2 * pi))
-        }
-        else # odd length
-        {
-            thetas <- 2 * pi * runif(n2)
-            angles <- c(0, thetas, -rev(thetas))
-            recf <- amplitudes * exp(complex(imaginary = angles))
-        }
-        temp <- Re(fft(recf, inverse = TRUE) / n)
-        
-        # adjust variance of the surrogate time series to match original
-        return(temp / sd(temp) * sigma)
+            if (n %% 2 == 0) # even length
+            {
+                thetas <- 2 * pi * runif(n2 - 1)
+                angles <- c(0, thetas, 0, -rev(thetas))
+                recf <- amplitudes * exp(complex(imaginary = angles))
+                recf[n2] <- complex(real = sqrt(2) * amplitudes[n2] * 
+                                        cos(runif(1) * 2 * pi))
+            }
+            else # odd length
+            {
+                thetas <- 2 * pi * runif(n2)
+                angles <- c(0, thetas, -rev(thetas))
+                recf <- amplitudes * exp(complex(imaginary = angles))
+            }
+            temp <- Re(fft(recf, inverse = TRUE) / n)
+            
+            # adjust variance of the surrogate time series to match original
+            return(temp / sd(temp) * sigma)
         })
     ), ncol = num_surr)
 }
@@ -118,6 +128,10 @@ make_surrogate_ebisuzaki <- function(ts, num_surr = 100)
 #' 
 make_surrogate_seasonal <- function(ts, num_surr = 100, T_period = 12)
 {
+    if (is.data.frame(ts))
+    {
+        ts <- ts[[1]]
+    }
     
     if (any(!is.finite(ts)))
         stop("input time series contained invalid values")
@@ -163,7 +177,7 @@ make_surrogate_seasonal <- function(ts, num_surr = 100, T_period = 12)
 #' @inheritParams make_surrogate_twin
 #' @return A vector of the same length as the original time series (here, 
 #'   `length(twins)`) and containing the reordered time indices.
-#'   
+#' @noRd
 make_twin_idx <- function(twins, 
                           phase_lock = TRUE, 
                           T_period = 24, 
@@ -239,7 +253,7 @@ make_twin_idx <- function(twins,
 #'   threshold for twins
 #' @return A list of the twins. The list has length equal to the time 
 #'   series, and each element is a vector of the candidate twins. 
-#'   
+#' @noRd
 identify_twins <- function(block, 
                            phase_lock = TRUE, 
                            T_period = 24, 
@@ -276,7 +290,7 @@ identify_twins <- function(block,
                      "length(twins) = ", length(twins), "\n", 
                      "names(twins) = ", names(twins))
             }
-            break()
+            break
         }
     }
     
@@ -309,13 +323,18 @@ identify_twins <- function(block,
 #' make_surrogate_twin(rnorm(100, sd = 0.1) + sin(1:100 * pi / 6), 10)
 #' 
 make_surrogate_twin <- function(ts,
-                                 num_surr = 1,
-                                 dim = 1, tau = 1, 
-                                 phase_lock = TRUE, 
-                                 T_period = 24, 
-                                 initial_point = "same_season", 
-                                 ...)
+                                num_surr = 1,
+                                dim = 1, tau = 1, 
+                                phase_lock = TRUE, 
+                                T_period = 24, 
+                                initial_point = "same_season", 
+                                ...)
 {
+    if (is.data.frame(ts))
+    {
+        ts <- ts[[1]]
+    }
+    
     # generate time-lag embedding matrix
     if (dim > 1) {
         block <- rEDM::make_block(ts, max_lag = dim, tau = tau)
@@ -383,7 +402,7 @@ make_block <- function(block, t = NULL, max_lag = 3, tau = 1, lib = NULL,
                        restrict_to_lib = TRUE)
 {
     # be sure to convert block if input is a vector
-    if(is.vector(block))
+    if (is.vector(block))
         block <- matrix(block, ncol = 1)
     num_vars <- NCOL(block)
     num_rows <- NROW(block)
@@ -411,7 +430,7 @@ make_block <- function(block, t = NULL, max_lag = 3, tau = 1, lib = NULL,
     for (j in 1:num_vars)
     {
         ts <- block[, j]
-        if(is.list(ts))
+        if (is.list(ts))
         {
             ts <- unlist(ts)
         }
@@ -429,7 +448,7 @@ make_block <- function(block, t = NULL, max_lag = 3, tau = 1, lib = NULL,
                 # make sure we pad beginning of lib segments with tau x NAs
                 if (!is.null(lib))
                 {
-                    for (k in 1:NROW(lib))
+                    for (k in seq_len(NROW(lib)))
                     {
                         ts[lib[k, 1] - 1 + (1:tau)] <- NA
                     }
