@@ -15,7 +15,7 @@ public:
     // *** methods *** //
     void set_time(const NumericVector new_time);
     void set_time_series(const NumericVector data);
-    void set_norm_type(const int norm_type);
+    void set_norm(const double norm);
     void set_pred_type(const int pred_type);
     void set_lib(const NumericMatrix lib);
     void set_pred(const NumericMatrix pred);
@@ -23,7 +23,6 @@ public:
     void set_epsilon(const double new_epsilon);
     void set_params(const size_t new_E, const size_t new_tau, const int new_tp, const size_t new_nn);
     void set_theta(const double new_theta);
-    void set_p(const double new_p);
     void suppress_warnings();
     void save_smap_coefficients();
     void run();
